package com.topcoder.persistence.relation;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

/**
 * This is a concrete implementation of a relation. A relation consists of a tuple type which
 * indicates the attribute names and types a tuple belonging to the relation can have,
 * a set of tuple constraints indicating the conditions a tuple has to respect to be
 * a part of the tuple and a set of relation constraints indicating global conditions
 * that the entire set of tuples in the relation must respect.
 * 
 * The inherited abstract class provides storage and access functions for the tuple type,
 * tuple constraints and relation constraints. These are common to all relation
 * implementations.
 * 
 * This class implements the actual tuple storage and the abstract methods related to tuples.
 * The storage is done using a HashSet of Tuples. This provides very good performance
 * (O(1) for add, remove, including duplicate detection).
 * 
 * Thread safety:
 * 
 * The class is not thread safe because it has state that can mutate.
 * There are lots of modifiers methods and many methods need access to several attributes.
 * That's why synchronizing all methods is the only reasonable solution if thread safety
 * is desired.
 */
public class DefaultRelation extends Relation {
	
	/**
	 * The set of tuples.
	 * Constains Tuple elements (no nulls).
	 * Initialized with a new HashSet.
	 * Used by all methods.
	 */
	private Set tuples = new HashSet();

	/**
	 * Constructor.
	 * 
	 * Implementation details:
	 * 
	 * Calls super(name).
	 * 
	 * @param name the name
	 * 
	 * @throws IllegalArgumentException if the name is null, empty or all spaces
	 */
	public DefaultRelation(String name) throws IllegalArgumentException {
		super(name);
	}
	
	/**
	 * Constructor.
	 * 
	 * Implementation details:
	 * 
	 * Calls super(name, tupleType).
	 * 
	 * @param name the name
	 * @param tupleType the tuple type
	 * 
	 * @throws IllegalArgumentException if the name is null, empty or all spaces or if tupleType is null
	 */
	public DefaultRelation(String name, TupleType tupleType) throws IllegalArgumentException {
		super(name, tupleType);
	}

	/**
	 * Adds a tuple to the relation.
	 * If the tuple already exists, nothing is done.
	 * 
	 * All tuple constraints are evaluated for the tuple to see if it meets them.
	 * If checkRelationConstraints == true, all relation constraints are checked too.
	 * If any constraint fails, an exception is thrown and the tuple is not added.
	 * 
	 * Implementation details:
	 * 
	 * - checks if already exists
	 * - iterates all tuple constraints and verifies that the tuple meets them
	 * - adds the tuple to the set
	 * - iterates all relation constraints (only if checkRelationConstraints == true)
	 * - if StatefulRelationConstraint, call tupleAdded to see if the constraint is still met
	 * - if just a simple RelationConstraint, call validate to see if the constraint is met
	 * - if any constraint is not met, remove tuple from the set and throw exception
	 *  
	 * @param tuple the tuple to add
	 * 
	 * @return true if added, false if not (duplicate)
	 * 
	 * @throws IllegalArgumentException if the argument is null
	 * @throws EvaluationException in an error occurs while evaluating the constraint
	 * @throws ConstraintViolatedException if the relation violates a constraint
	 * @throws IllegalStateException if the tuple type has not been set yet
	 */
	public boolean addTuple(Tuple tuple) throws IllegalArgumentException, EvaluationException, ConstraintViolatedException, IllegalStateException {
		return false;
	}

	/**
	 * Removes a tuple from the relation.
	 * If the tuple doesn't exists, nothing is done.
	 * 
	 * If checkRelationConstraints == true, all relation constraints are checked.
	 * If any constraint fails, an exception is thrown and the tuple is not removed.
	 * 
	 * Implementation details:
	 * 
	 * - checks if exists in the set
	 * - removes the tuple to the set
	 * - iterates all relation constraints (only if checkRelationConstraints == true)
	 * - if StatefulRelationConstraint, call tupleRemoved to see if the constraint is still met
	 * - if just a simple RelationConstraint, call validate to see if the constraint is met
	 * - if any constraint is not met, add tuple back to the set and throw exception
	 * 
	 * @param tuple the tuple to remove
	 * 
	 * @return true if removed, false if not (not found)
	 * 
	 * @throws IllegalArgumentException if the argument is null
	 * @throws EvaluationException in an error occurs while evaluating the constraint
	 * @throws ConstraintViolatedException if the relation violates a constraint
	 */
	public boolean removeTuple(Tuple tuple) throws IllegalArgumentException, EvaluationException, ConstraintViolatedException {
		return false;
	}
	
	/**
	 * Removes all tuples from the relation.
	 * 
	 * If checkRelationConstraints == true, all relation constraints are checked.
	 * If any constraint fails, an exception is thrown and the clear is not performed.
	 * 
	 * Implementation details:
	 * 
	 * - replace the set with an empty set
	 * - iterates all relation constraints (only if checkRelationConstraints == true)
	 * - call validate on the constraint to see if the constraint is met
	 * - if any constraint is not met, replace the empty set with the old set and throw exception
	 * 
	 * @throws EvaluationException in an error occurs while evaluating the constraint
	 * @throws ConstraintViolatedException if the relation violates a constraint
	 */
	public void clearTuples() throws EvaluationException, ConstraintViolatedException {
	}
	
    /**
     * Returns the tuple count.
     * 
	 * Implementation details:
	 * 
	 * Returns the size of the set.
	 * 
     * @return the tuple count.
     */
	public int getTupleCount() {
		return 0;
	}
	
	/**
	 * Returns all tuples from the relation.
	 * 
	 * Implementation details:
	 * 
	 * Returns an array with all tuples from the set.
	 * 
	 * @return all tuples from the relation
	 */
	public Tuple[] getAllTuples() {
		return null;
	}
	
	/**
	 * Returns an iterator for the tuples from the relation.
	 * This is meant to be an efficient way to check the tuples.
	 * The array returning version is not that efficient because an array has to be created. 
	 * 
	 * Implementation details:
	 * 
	 * Returns an iterator over the set.
	 * 
	 * @return the tuple iterator
	 */	
	public Iterator getTuplesIterator() {
		return null;
	}
	
    /**
     * Basic find method that finds tuples by an attribute value.
     * For anything that is more complex, probably the restrict operation is a better choice.
     * 
	 * Implementation details:
	 * 
	 * Iterates all tuples and select those having the given value for the attribute.
     * 
     * @param attribute the attribute to find by
     * @param value the value of the attribute
     * 
     * @return the found tuples
     * 
     * @throws IllegalArgumentException if any argument is null or if the string is empty or all spaces
     */	
	public Tuple[] findTuples(String attribute, Object value) throws IllegalArgumentException {
		return null;
	}
	
    /**
     * Basic find method that finds tuples by some attribute values.
     * For anything that is more complex, probably the restrict operation is a better choice.
     * 
	 * Implementation details:
	 * 
	 * Iterates all tuples and select those having the given values for the attributes. 
     * 
     * @param attributes the attributes to find by
     * @param values the values of the attributes
     * 
     * @return the found tuples
     * 
     * @throws IllegalArgumentException if any argument is null, if the arrays have different size or
     * if they have null elements, empty or all spaces strings
     */	
	public Tuple[] findTuples(String attributes[], Object values[]) throws IllegalArgumentException {
		return null;
	}

}
