package com.topcoder.persistence.relation.operations;

import com.topcoder.persistence.relation.OperationException;
import com.topcoder.persistence.relation.Relation;

/**
 * A natural join implementation for two relations.
 * This is a special type of join that considers that two tuples from
 * the two relations match if all common attributes have the same values.
 * 
 * This type of join is a special case of equi-join (EqualsJoinOperation).
 * The only difference is that the join attributes are all common attributes
 * (in equals join the user has to explicitely mention the join attributes).
 *
 * This join is implemented trivially by finding the list of common attributes
 * and calling the EqualsJoinOperation.compute method to take care of the rest.
 *
 * The result relation is given by the user (must be empty and typeless).
 * This allows the user to select the desired relation implementation
 * (optimized for speed, for memory, etc).
 * 
 * Thread safety:
 * 
 * This class is immutable so it is thread safe.
 * 
 * @author TCSDESIGNER
 */
public class NaturalJoinOperation extends EqualsJoinOperation {

	/**
	 * Default constructor that doesn't do anything.
	 */
	public NaturalJoinOperation() {
	}

	/**
	 * Computes the natural join of two relations.
	 * The natural join is a join in which all common attributes from the two relations must match.
	 * 
	 * Implementation details:
	 * 
	 * The natural join algorithm is just a special case of equals join (equi-join).
	 * The only difference is that the join attributes are all common attributes
	 * (in equals join the user has to explicitely mention the join attributes).
	 * 
	 * Just as the equals join, the complexity is O(n + max(m, result_size)).
	 * 
	 * - finds all common attributes of the two relations
	 * - calls setJoinAttributes and passes the common attributes
	 * - calls super.compute (EqualsJoinOperator)  
	 *  
	 * @param leftRelation the left relation
	 * @param rightRelation the right relation
	 * @param resultRelation the result relation (must be empty and typeless)
	 * 
	 * @throws IllegalArgumentException if any argument is null or the result relation is not empty and typeless
	 * @throws OperationException if any of the checks in the join algorithm fails
	 */
	public void compute(Relation leftRelation, Relation rightRelation, Relation resultRelation) throws IllegalArgumentException, OperationException {
	}

}
